# WARNING: Please don't edit this file. It was generated by Python/WinRT v3.1.0

import winrt.system
from winrt._winrt_windows_graphics import (
    DisplayAdapterId,
    DisplayId,
    PointInt32,
    RectInt32,
    SizeInt32,
    _IGeometrySource2D,
    IGeometrySource2D,
)

__all__ = [
    "DisplayAdapterId",
    "DisplayId",
    "PointInt32",
    "RectInt32",
    "SizeInt32",
    "IGeometrySource2D",
]

